import 'package:http/http.dart' as http;
import 'dart:io';
import 'dart:async';

enum CheckMethod {
  OPTIONS,
  HEAD,
}

class ConnectionUtils {
  static Future<http.Response> options(String url,
      {Map<String, String>? headers, http.Client? httpClient}) async {
    final client = httpClient ?? http.Client();
    final request = http.Request('OPTIONS', Uri.parse(url));
    if (headers != null) request.headers.addAll(headers);
    final streamedResponse = await client.send(request);
    return http.Response.fromStream(streamedResponse);
  }

  static Future<http.Response> head(String url,
      {Map<String, String>? headers, http.Client? httpClient}) {
    final client = httpClient ?? http.Client();
    return client.head(Uri.parse(url), headers: headers);
  }

  static Future<bool> isResponseSuccessful(String url,
      {Duration? timeout,
      int? timeoutMillis,
      CheckMethod checkMethod = CheckMethod.OPTIONS,
      Map<String, String>? headers,
      http.Client? httpClient}) async {
    final future = checkMethod == CheckMethod.OPTIONS
        ? options(url, headers: headers, httpClient: httpClient)
        : head(url, headers: headers, httpClient: httpClient);
    if (timeoutMillis != null) timeout = Duration(milliseconds: timeoutMillis);
    final response = await (timeout == null
        ? future
        : future.timeout(timeout,
            onTimeout: () => http.Response('', HttpStatus.requestTimeout)));
    return response.statusCode >= 200 && response.statusCode < 300;
  }

  static Future<bool> isResponseOnline(String url,
      {Duration? timeout,
      int? timeoutMillis,
      CheckMethod checkMethod = CheckMethod.OPTIONS,
      Map<String, String>? headers,
      http.Client? httpClient}) async {
    final future = checkMethod == CheckMethod.OPTIONS
        ? options(url, headers: headers, httpClient: httpClient)
        : head(url, headers: headers, httpClient: httpClient);
    if (timeoutMillis != null) timeout = Duration(milliseconds: timeoutMillis);
    final response = await (timeout == null
        ? future
        : future.timeout(timeout,
            onTimeout: () => http.Response('', HttpStatus.requestTimeout)));
    switch (response.statusCode) {
      case HttpStatus.notFound:
      case HttpStatus.requestTimeout:
      case HttpStatus.gatewayTimeout:
      case HttpStatus.networkConnectTimeoutError:
        return false;
    }
    return true;
  }
}
